import { CorTabComponent } from './cor-tab.component';
import { CorTabPanelComponent } from './cor-tab-panel.component';

/**
 * Defines an interface for reading and writing the current tab state.
 */
export interface CorTabCurrentHandler {
  getInitialTabId(): string

  notifyTabChanged(tab: CorTabComponent, isDefaultTab: boolean)

  dispose(): void
}

export function CorTabCurrentHandlerFactory(options?: any): CorTabCurrentHandler {
  switch (options.type) {
    case "cookie":
      return new CookieCurrentTabHandler(options.cookieService, options.cookieName);
    default:
      return new LocationCurrentTabHandler(options.panel, options.$location, options.$rootScope);
  }
}

/**
 * Reads and writes the tab from the `tab` query parameter in the location.
 */
export class LocationCurrentTabHandler implements CorTabCurrentHandler {
  private cancelWatchHandle: Function;

  constructor (private panel: CorTabPanelComponent,
               private $location: ng.ILocationService,
               private $rootScope: ng.IRootScopeService) {
  }

  private checkLocation(): void {
    var specifiedTabId = this.$location.search()['tab'];
    var specifiedTab = this.panel.findTab(specifiedTabId);
    this.panel.setActiveTab(specifiedTab);
  }

  public getInitialTabId(): string {
    if (!this.cancelWatchHandle) {
      this.cancelWatchHandle = this.$rootScope.$on('$routeUpdate', () => this.checkLocation());
    }
    return this.$location.search()['tab'];
  }

  public notifyTabChanged(tab: CorTabComponent, isDefaultTab: boolean) {
    var newSearch = $.extend(this.$location.search(), {});
    if (isDefaultTab) {
      delete newSearch['tab'];
    } else {
      newSearch['tab'] = tab.tabId;
    }

    this.$location.search(newSearch);
  }

  public dispose(): void {
    this.cancelWatchHandle();
  }
}

/**
 * Reads and writes the tab from a cookie,.
 */
export class CookieCurrentTabHandler  implements CorTabCurrentHandler {
  constructor (private CookieService: any, private cookieName: string) {}

  public getInitialTabId(): string {
    return this.CookieService.get(this.cookieName);
  }

  public notifyTabChanged(tab: CorTabComponent, isDefaultTab: boolean) {
    if (isDefaultTab) {
      this.CookieService.clear(this.cookieName);
    } else {
      this.CookieService.putPermanent(this.cookieName, tab.tabId);
    }
  }

  public dispose(): void {}
}