import { Component, Input, Inject, OnDestroy } from 'ng-metadata/core';
import { CorTabComponent } from './cor-tab.component';
import { CorTabPaneComponent } from './cor-tab-pane.component';
import { CorTabCurrentHandler, LocationCurrentTabHandler, CookieCurrentTabHandler, CorTabCurrentHandlerFactory } from './cor-tab-handlers'


/**
 * A component that contains a cor-tabs and handles all of its logic.
 */
@Component({
  selector: 'cor-tab-panel',
  templateUrl: '/static/js/directives/ui/cor-tabs/cor-tab-panel.component.html',
  legacy: {
    transclude: true,
  }
})
export class CorTabPanelComponent implements OnDestroy {
  // If supplied, the currently selected tab will be remembered via the named cookie and not
  // the page URL.
  @Input('@') public rememberCookie: string;

  // If 'true', the tabs will be displayed vertically, as opposed to horizontally.
  @Input('<') public verticalTabs: boolean;

  // The tabs under this tabs component.
  private tabs: CorTabComponent[] = [];

  // The tab panes under the tabs component, indexed by the tab id.
  private tabPanes: {[id: string]: CorTabPaneComponent} = {};

  // The currently active tab, if any.
  private activeTab: CorTabComponent = null;

  // Whether the initial tab was set.
  private initialTabSet: boolean = false;

  // The handler to use to read/write the current tab.
  private currentTabHandler: CorTabCurrentHandler = null;

  constructor(@Inject('$location') private $location: ng.ILocationService,
              @Inject('$rootScope') private $rootScope: ng.IRootScopeService,
              @Inject('CookieService') private CookieService: any,
              @Inject('CorTabCurrentHandlerFactory') private CorTabCurrentHandlerFactory: (Object) => CorTabCurrentHandler) {
  }

  public ngOnDestroy(): void {
    if (this.currentTabHandler) {
      this.currentTabHandler.dispose();
    }
  }

  /**
   * isVertical returns true if the tabs in this panel are displayed vertically.
   */
  public isVertical(): boolean {
    return this.verticalTabs;
  }

  public tabClicked(tab: CorTabComponent): void {
    this.setActiveTab(tab);
  }

  public findTab(tabId: string): CorTabComponent {
    if (!this.tabs.length) {
      return null;
    }

    var tab = this.tabs.find(function(current) {
      return current.tabId == tabId;
    }) || this.tabs[0];

    if (!this.tabPanes[tab.tabId]) {
      return null;
    }

    return tab;
  }

  public setActiveTab(tab: CorTabComponent): void {
    if (this.activeTab == tab) {
      return;
    }

    if (this.activeTab != null) {
      this.activeTab.changeState(false);
      this.tabPanes[this.activeTab.tabId].changeState(false);
    }

    this.activeTab = tab;
    this.activeTab.changeState(true);
    this.tabPanes[this.activeTab.tabId].changeState(true);
    this.currentTabHandler.notifyTabChanged(tab, this.tabs[0] == tab);
  }

  public addTab(tab: CorTabComponent): void {
    this.tabs.push(tab);
    this.checkInitialTab();
  }

  public addTabPane(tabPane: CorTabPaneComponent): void {
    this.tabPanes[tabPane.id] = tabPane;
    this.checkInitialTab();
  }

  private checkInitialTab(): void {
    if (this.tabs.length < 1 || this.initialTabSet) {
      return;
    }

    this.currentTabHandler = this.CorTabCurrentHandlerFactory({
      type: this.rememberCookie ? 'cookie' : 'location',
      cookieService: this.CookieService,
      cookeName: this.rememberCookie,
      panel: this,
      $location: this.$location,
      $rootScope: this.$rootScope,
    });

    var tabId = this.currentTabHandler.getInitialTabId();
    var tab = this.findTab(tabId);
    if (!tab) {
      return;
    }

    this.initialTabSet = true;
    this.setActiveTab(tab);
  }
}
